import kotlinx.browser.document
import kotlinx.browser.window
import kotlinx.html.*
import kotlinx.html.dom.append
import kotlinx.html.js.div
import kotlinx.html.js.onClickFunction
import kotlinx.html.js.onKeyUpFunction
import org.w3c.dom.HTMLDivElement
import org.w3c.dom.HTMLTextAreaElement
import org.w3c.dom.get

external fun callMain(argument: Array<out String>)

fun ybcon(vararg argument: String) = callMain(argument)

val console get() = document.body!!.children["console"] as HTMLDivElement

private var typingTimer: Int = 0
const val typing_interval = 500

@JsExport
fun makeConsole() {
    with(document.body!!.append) {
        div {
            id = "console"
            style { unsafe { raw(
                """
                    #console {
                        position: relative;
                        display: none; filter: blur(0px);
                        transition: all 0.25s ease-in;
                    }
                    #console:hover { box-shadow: 0 0 50px #2f5361; }
                    #console, #input_box { border-bottom-left-radius: 10px; border-top-left-radius: 10px; }
                    #console, #output_box { border-bottom-right-radius: 10px; border-top-right-radius: 10px; }
                    textArea {
                        color: white;
                        border-color: #2f5361;
                        outline: none !important;
                        background: #2f5361;
                        font-family:"Noto Sans Mono SemiBold", Noto, mono;
                        width: 25vw; height: 50vh;
                        resize: none;
                    }
                """.trimIndent()
            ) } }
            script(src = "/ybcon.js") {} // Note: ybcon.js and ybcon.worker.js have to be available at this path on the server
            textArea {
                id = "input_box"
                onKeyUpFunction = { event ->
                    window.clearTimeout(typingTimer)
                    ybcon("-t", (event.target!! as HTMLTextAreaElement).value)
                    typingTimer = window.setTimeout({
                        (console.children["output_box"] as HTMLTextAreaElement).value = js("yerbacon_output") as String
                    }, typing_interval)
                }
            }
            textArea {
                readonly = true
                id = "output_box"
            }
        }
        div {
            style { unsafe {
                raw("""
                button {
                    position: fixed;
                    bottom: 10px;
                    right: 10px;
                    border: none;
                    background: inherit;
                }
                button:hover {
                    background: #2f5361;
                }
                """.trimIndent())
            } }
            button {
                + "Yerbacon programming language project"
                onClickFunction = {
                    with(console.style) {
                        display = when (display) {
                            "none", "" -> "inline-block"
                            else -> "none"
                        }
                    }
                }
            }
        }
    }
}

fun main() {
    if (window.closed) window.open("index.html")
}
